@@ -21,11 +21,10 @@
  */
 
 #include "config.h"
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
+#include "glib.h"
+#ifdef G_OS_UNIX
+#include "glib-unix.h"
 #endif
-#include <errno.h>
-#include <fcntl.h>
 #include <gioerror.h>
 #ifdef G_OS_WIN32
 #include <windows.h>
@@ -193,36 +192,6 @@ g_cancellable_class_init (GCancellableClass *klass)
 }
 
 #ifndef G_OS_WIN32
-static void
-set_fd_nonblocking (int fd)
-{
-#ifdef F_GETFL
-  glong fcntl_flags;
-  fcntl_flags = fcntl (fd, F_GETFL);
-
-#ifdef O_NONBLOCK
-  fcntl_flags |= O_NONBLOCK;
-#else
-  fcntl_flags |= O_NDELAY;
-#endif
-
-  fcntl (fd, F_SETFL, fcntl_flags);
-#endif
-}
-
-static void
-set_fd_close_exec (int fd)
-{
-  int flags;
-
-  flags = fcntl (fd, F_GETFD, 0);
-  if (flags != -1 && (flags & FD_CLOEXEC) == 0)
-    {
-      flags |= FD_CLOEXEC;
-      fcntl (fd, F_SETFD, flags);
-    }
-}
-
 
 static void
 g_cancellable_open_pipe (GCancellable *cancellable)
@@ -230,15 +199,13 @@ g_cancellable_open_pipe (GCancellable *cancellable)
   GCancellablePrivate *priv;
 
   priv = cancellable->priv;
-  if (pipe (priv->cancel_pipe) == 0)
+  if (g_unix_open_pipe (priv->cancel_pipe, FD_CLOEXEC, NULL))
     {
       /* Make them nonblocking, just to be sure we don't block
        * on errors and stuff
        */
-      set_fd_nonblocking (priv->cancel_pipe[0]);
-      set_fd_nonblocking (priv->cancel_pipe[1]);
-      set_fd_close_exec (priv->cancel_pipe[0]);
-      set_fd_close_exec (priv->cancel_pipe[1]);
+      g_unix_set_fd_nonblocking (priv->cancel_pipe[0], TRUE, NULL);
+      g_unix_set_fd_nonblocking (priv->cancel_pipe[1], TRUE, NULL);
       
       if (priv->cancelled)
         {
