@@ -38,12 +38,13 @@
 #include "gasyncresult.h"
 #include "gsimpleasyncresult.h"
 #include "gcancellable.h"
+#include "gdbusinterface.h"
 
 #include "glibintl.h"
 
 /**
  * SECTION:gdbusproxy
- * @short_description: Client-side proxies
+ * @short_description: Client-side D-Bus interface proxy
  * @include: gio/gio.h
  *
  * #GDBusProxy is a base class used for proxies to access a D-Bus
@@ -82,9 +83,9 @@
 struct _GDBusProxyPrivate
 {
   GBusType bus_type;
+  GDBusProxyFlags flags;
   GDBusConnection *connection;
 
-  GDBusProxyFlags flags;
   gchar *name;
   gchar *name_owner;
   gchar *object_path;
@@ -104,6 +105,8 @@ struct _GDBusProxyPrivate
   guint signals_subscriber_id;
 
   gboolean initialized;
+
+  GDBusObject *object;
 };
 
 enum
@@ -129,10 +132,12 @@ enum
 
 guint signals[LAST_SIGNAL] = {0};
 
+static void dbus_interface_iface_init (GDBusInterfaceIface *dbus_interface_iface);
 static void initable_iface_init       (GInitableIface *initable_iface);
 static void async_initable_iface_init (GAsyncInitableIface *async_initable_iface);
 
 G_DEFINE_TYPE_WITH_CODE (GDBusProxy, g_dbus_proxy, G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (G_TYPE_DBUS_INTERFACE, dbus_interface_iface_init)
                          G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE, initable_iface_init)
                          G_IMPLEMENT_INTERFACE (G_TYPE_ASYNC_INITABLE, async_initable_iface_init)
                          );
@@ -156,7 +161,8 @@ g_dbus_proxy_finalize (GObject *object)
     g_dbus_connection_signal_unsubscribe (proxy->priv->connection,
                                           proxy->priv->signals_subscriber_id);
 
-  g_object_unref (proxy->priv->connection);
+  if (proxy->priv->connection != NULL)
+    g_object_unref (proxy->priv->connection);
   g_free (proxy->priv->name);
   g_free (proxy->priv->name_owner);
   g_free (proxy->priv->object_path);
@@ -165,7 +171,13 @@ g_dbus_proxy_finalize (GObject *object)
     g_hash_table_unref (proxy->priv->properties);
 
   if (proxy->priv->expected_interface != NULL)
-    g_dbus_interface_info_unref (proxy->priv->expected_interface);
+    {
+      g_dbus_interface_info_cache_release (proxy->priv->expected_interface);
+      g_dbus_interface_info_unref (proxy->priv->expected_interface);
+    }
+
+  if (proxy->priv->object != NULL)
+    g_object_remove_weak_pointer (G_OBJECT (proxy->priv->object), (gpointer *) &proxy->priv->object);
 
   G_OBJECT_CLASS (g_dbus_proxy_parent_class)->finalize (object);
 }
@@ -493,7 +505,7 @@ g_dbus_proxy_class_init (GDBusProxyClass *klass)
    */
   signals[PROPERTIES_CHANGED_SIGNAL] = g_signal_new ("g-properties-changed",
                                                      G_TYPE_DBUS_PROXY,
-                                                     G_SIGNAL_RUN_LAST,
+                                                     G_SIGNAL_RUN_LAST | G_SIGNAL_MUST_COLLECT,
                                                      G_STRUCT_OFFSET (GDBusProxyClass, g_properties_changed),
                                                      NULL,
                                                      NULL,
@@ -516,7 +528,7 @@ g_dbus_proxy_class_init (GDBusProxyClass *klass)
    */
   signals[SIGNAL_SIGNAL] = g_signal_new ("g-signal",
                                          G_TYPE_DBUS_PROXY,
-                                         G_SIGNAL_RUN_LAST,
+                                         G_SIGNAL_RUN_LAST | G_SIGNAL_MUST_COLLECT,
                                          G_STRUCT_OFFSET (GDBusProxyClass, g_signal),
                                          NULL,
                                          NULL,
@@ -639,8 +651,7 @@ g_dbus_proxy_get_cached_property (GDBusProxy   *proxy,
   value = g_hash_table_lookup (proxy->priv->properties, property_name);
   if (value == NULL)
     {
-      const GDBusPropertyInfo *info;
-      info = lookup_property_info_or_warn (proxy, property_name);
+      lookup_property_info_or_warn (proxy, property_name);
       /* no difference */
       goto out;
     }
@@ -655,7 +666,7 @@ g_dbus_proxy_get_cached_property (GDBusProxy   *proxy,
  * g_dbus_proxy_set_cached_property:
  * @proxy: A #GDBusProxy
  * @property_name: Property name.
- * @value: Value for the property or %NULL to remove it from the cache.
+ * @value: (allow-none): Value for the property or %NULL to remove it from the cache.
  *
  * If @value is not %NULL, sets the cached value for the property with
  * name @property_name to the value in @value.
@@ -711,8 +722,8 @@ g_dbus_proxy_set_cached_property (GDBusProxy   *proxy,
         {
           if (g_strcmp0 (info->signature, g_variant_get_type_string (value)) != 0)
             {
-              g_warning (_("Trying to set property %s of type %s but according to the expected "
-                           "interface the type is %s"),
+              g_warning ("Trying to set property %s of type %s but according to the expected "
+			 "interface the type is %s",
                          property_name,
                          g_variant_get_type_string (value),
                          info->signature);
@@ -751,6 +762,22 @@ on_signal_received (GDBusConnection *connection,
   if (proxy->priv->name_owner != NULL && g_strcmp0 (sender_name, proxy->priv->name_owner) != 0)
     goto out;
 
+  if (proxy->priv->expected_interface != NULL)
+    {
+      const GDBusSignalInfo *info;
+      GVariantType *expected_type;
+      info = g_dbus_interface_info_lookup_signal (proxy->priv->expected_interface, signal_name);
+      if (info == NULL)
+        goto out;
+      expected_type = _g_dbus_compute_complete_signature (info->args);
+      if (!g_variant_type_equal (expected_type, g_variant_get_type (parameters)))
+        {
+          g_variant_type_free (expected_type);
+          goto out;
+        }
+      g_variant_type_free (expected_type);
+    }
+
   g_signal_emit (proxy,
                  signals[SIGNAL_SIGNAL],
                  0,
@@ -763,6 +790,36 @@ on_signal_received (GDBusConnection *connection,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static void
+insert_property_checked (GDBusProxy  *proxy,
+			 gchar *property_name,
+			 GVariant *value)
+{
+  if (proxy->priv->expected_interface != NULL)
+    {
+      const GDBusPropertyInfo *info;
+
+      info = g_dbus_interface_info_lookup_property (proxy->priv->expected_interface, property_name);
+      /* Ignore unknown properties */
+      if (info == NULL)
+	goto invalid;
+
+      /* Ignore properties with the wrong type */
+      if (g_strcmp0 (info->signature, g_variant_get_type_string (value)) != 0)
+	goto invalid;
+    }
+
+  g_hash_table_insert (proxy->priv->properties,
+		       property_name, /* adopts string */
+		       value); /* adopts value */
+
+  return;
+
+ invalid:
+  g_variant_unref (value);
+  g_free (property_name);
+}
+
 static void
 on_properties_changed (GDBusConnection *connection,
                        const gchar     *sender_name,
@@ -773,7 +830,6 @@ on_properties_changed (GDBusConnection *connection,
                        gpointer         user_data)
 {
   GDBusProxy *proxy = G_DBUS_PROXY (user_data);
-  GError *error;
   const gchar *interface_name_for_signal;
   GVariant *changed_properties;
   gchar **invalidated_properties;
@@ -782,7 +838,6 @@ on_properties_changed (GDBusConnection *connection,
   GVariant *value;
   guint n;
 
-  error = NULL;
   changed_properties = NULL;
   invalidated_properties = NULL;
 
@@ -811,9 +866,9 @@ on_properties_changed (GDBusConnection *connection,
   g_variant_iter_init (&iter, changed_properties);
   while (g_variant_iter_next (&iter, "{sv}", &key, &value))
     {
-      g_hash_table_insert (proxy->priv->properties,
-                           key, /* adopts string */
-                           value); /* adopts value */
+      insert_property_checked (proxy,
+			       key, /* adopts string */
+			       value); /* adopts value */
     }
 
   for (n = 0; invalidated_properties[n] != NULL; n++)
@@ -853,9 +908,9 @@ process_get_all_reply (GDBusProxy *proxy,
   g_variant_get (result, "(a{sv})", &iter);
   while (g_variant_iter_next (iter, "{sv}", &key, &value))
     {
-      g_hash_table_insert (proxy->priv->properties,
-                           key, /* adopts string */
-                           value); /* adopts value */
+      insert_property_checked (proxy,
+			       key, /* adopts string */
+			       value); /* adopts value */
     }
   g_variant_iter_free (iter);
 
@@ -982,14 +1037,13 @@ on_name_owner_changed (GDBusConnection *connection,
           g_hash_table_size (proxy->priv->properties) > 0)
         {
           GVariantBuilder builder;
-          GVariant *changed_properties;
           GPtrArray *invalidated_properties;
           GHashTableIter iter;
           const gchar *key;
 
           /* Build changed_properties (always empty) and invalidated_properties ... */
           g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
-          changed_properties = g_variant_builder_end (&builder);
+
           invalidated_properties = g_ptr_array_new_with_free_func (g_free);
           g_hash_table_iter_init (&iter, proxy->priv->properties);
           while (g_hash_table_iter_next (&iter, (gpointer) &key, NULL))
@@ -1002,9 +1056,8 @@ on_name_owner_changed (GDBusConnection *connection,
           /* ... and finally emit the ::g-properties-changed signal */
           g_signal_emit (proxy, signals[PROPERTIES_CHANGED_SIGNAL],
                          0,
-                         changed_properties,
+                         g_variant_builder_end (&builder) /* consumed */,
                          (const gchar* const *) invalidated_properties->pdata);
-          g_variant_unref (changed_properties);
           g_ptr_array_unref (invalidated_properties);
         }
       g_object_notify (G_OBJECT (proxy), "g-name-owner");
@@ -1140,8 +1193,7 @@ async_init_get_name_owner_cb (GDBusConnection *connection,
             }
           else
             {
-              g_simple_async_result_set_from_error (data->simple, error);
-              g_error_free (error);
+              g_simple_async_result_take_error (data->simple, error);
               g_simple_async_result_complete_in_idle (data->simple);
               async_init_data_free (data);
               goto out;
@@ -1265,8 +1317,7 @@ async_init_start_service_by_name_cb (GDBusConnection *connection,
 
  failed:
   g_warn_if_fail (error != NULL);
-  g_simple_async_result_set_from_error (data->simple, error);
-  g_error_free (error);
+  g_simple_async_result_take_error (data->simple, error);
   g_simple_async_result_complete_in_idle (data->simple);
   async_init_data_free (data);
 }
@@ -1453,8 +1504,7 @@ get_connection_cb (GObject       *source_object,
                                           data->callback,
                                           data->user_data,
                                           NULL);
-      g_simple_async_result_set_from_error (simple, error);
-      g_error_free (error);
+      g_simple_async_result_take_error (simple, error);
       g_simple_async_result_complete_in_idle (simple);
       g_object_unref (simple);
     }
@@ -1615,8 +1665,8 @@ initable_iface_init (GInitableIface *initable_iface)
  * g_dbus_proxy_new:
  * @connection: A #GDBusConnection.
  * @flags: Flags used when constructing the proxy.
- * @info: A #GDBusInterfaceInfo specifying the minimal interface that @proxy conforms to or %NULL.
- * @name: A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
+ * @info: (allow-none): A #GDBusInterfaceInfo specifying the minimal interface that @proxy conforms to or %NULL.
+ * @name: (allow-none): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
  * @object_path: An object path.
  * @interface_name: A D-Bus interface name.
  * @cancellable: A #GCancellable or %NULL.
@@ -1785,7 +1835,7 @@ g_dbus_proxy_new_sync (GDBusConnection     *connection,
  * g_dbus_proxy_new_for_bus:
  * @bus_type: A #GBusType.
  * @flags: Flags used when constructing the proxy.
- * @info: A #GDBusInterfaceInfo specifying the minimal interface that @proxy conforms to or %NULL.
+ * @info: (allow-none): A #GDBusInterfaceInfo specifying the minimal interface that @proxy conforms to or %NULL.
  * @name: A bus name (well-known or unique).
  * @object_path: An object path.
  * @interface_name: A D-Bus interface name.
@@ -1850,7 +1900,8 @@ g_dbus_proxy_new_for_bus_finish (GAsyncResult  *res,
  * g_dbus_proxy_new_for_bus_sync:
  * @bus_type: A #GBusType.
  * @flags: Flags used when constructing the proxy.
- * @info: A #GDBusInterfaceInfo specifying the minimal interface that @proxy conforms to or %NULL.
+ * @info: (allow-none): A #GDBusInterfaceInfo specifying the minimal interface
+ *        that @proxy conforms to or %NULL.
  * @name: A bus name (well-known or unique).
  * @object_path: An object path.
  * @interface_name: A D-Bus interface name.
@@ -2077,7 +2128,7 @@ g_dbus_proxy_get_interface_info (GDBusProxy *proxy)
 /**
  * g_dbus_proxy_set_interface_info:
  * @proxy: A #GDBusProxy
- * @info: Minimum interface this proxy conforms to or %NULL to unset.
+ * @info: (allow-none): Minimum interface this proxy conforms to or %NULL to unset.
  *
  * Ensure that interactions with @proxy conform to the given
  * interface.  For example, when completing a method call, if the type
@@ -2095,8 +2146,13 @@ g_dbus_proxy_set_interface_info (GDBusProxy         *proxy,
 {
   g_return_if_fail (G_IS_DBUS_PROXY (proxy));
   if (proxy->priv->expected_interface != NULL)
-    g_dbus_interface_info_unref (proxy->priv->expected_interface);
+    {
+      g_dbus_interface_info_cache_release (proxy->priv->expected_interface);
+      g_dbus_interface_info_unref (proxy->priv->expected_interface);
+    }
   proxy->priv->expected_interface = info != NULL ? g_dbus_interface_info_ref (info) : NULL;
+  if (proxy->priv->expected_interface != NULL)
+    g_dbus_interface_info_cache_build (proxy->priv->expected_interface);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -2148,9 +2204,7 @@ reply_cb (GDBusConnection *connection,
                                          &error);
   if (error != NULL)
     {
-      g_simple_async_result_set_from_error (simple,
-                                            error);
-      g_error_free (error);
+      g_simple_async_result_take_error (simple, error);
     }
   else
     {
@@ -2212,7 +2266,7 @@ get_destination_for_call (GDBusProxy *proxy)
  * g_dbus_proxy_call:
  * @proxy: A #GDBusProxy.
  * @method_name: Name of method to invoke.
- * @parameters: A #GVariant tuple with parameters for the signal or %NULL if not passing parameters.
+ * @parameters: (allow-none): A #GVariant tuple with parameters for the signal or %NULL if not passing parameters.
  * @flags: Flags from the #GDBusCallFlags enumeration.
  * @timeout_msec: The timeout in milliseconds (with %G_MAXINT meaning
  *                "infinite") or -1 to use the proxy default timeout.
@@ -2380,7 +2434,8 @@ g_dbus_proxy_call_finish (GDBusProxy    *proxy,
  * g_dbus_proxy_call_sync:
  * @proxy: A #GDBusProxy.
  * @method_name: Name of method to invoke.
- * @parameters: A #GVariant tuple with parameters for the signal or %NULL if not passing parameters.
+ * @parameters: (allow-none): A #GVariant tuple with parameters for the signal
+ *              or %NULL if not passing parameters.
  * @flags: Flags from the #GDBusCallFlags enumeration.
  * @timeout_msec: The timeout in milliseconds (with %G_MAXINT meaning
  *                "infinite") or -1 to use the proxy default timeout.
@@ -2499,3 +2554,39 @@ g_dbus_proxy_call_sync (GDBusProxy      *proxy,
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
+
+static GDBusInterfaceInfo *
+_g_dbus_proxy_get_info (GDBusInterface *interface)
+{
+  GDBusProxy *proxy = G_DBUS_PROXY (interface);
+  return g_dbus_proxy_get_interface_info (proxy);
+}
+
+static GDBusObject *
+_g_dbus_proxy_get_object (GDBusInterface *interface)
+{
+  GDBusProxy *proxy = G_DBUS_PROXY (interface);
+  return proxy->priv->object;
+}
+
+static void
+_g_dbus_proxy_set_object (GDBusInterface *interface,
+                          GDBusObject    *object)
+{
+  GDBusProxy *proxy = G_DBUS_PROXY (interface);
+  if (proxy->priv->object != NULL)
+    g_object_remove_weak_pointer (G_OBJECT (proxy->priv->object), (gpointer *) &proxy->priv->object);
+  proxy->priv->object = object;
+  if (proxy->priv->object != NULL)
+    g_object_add_weak_pointer (G_OBJECT (proxy->priv->object), (gpointer *) &proxy->priv->object);
+}
+
+static void
+dbus_interface_iface_init (GDBusInterfaceIface *dbus_interface_iface)
+{
+  dbus_interface_iface->get_info   = _g_dbus_proxy_get_info;
+  dbus_interface_iface->get_object = _g_dbus_proxy_get_object;
+  dbus_interface_iface->set_object = _g_dbus_proxy_set_object;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
