@@ -20,19 +20,13 @@
 
 #include "config.h"
 
-#define _GNU_SOURCE
-
 /* We want to build the fork-based version, not the threaded version. */
 #undef HAVE_PTHREAD
 
-/* For old OS X, #580301. Remove if these are added to asyncns.c
- * in the future.
- */
-#include <sys/time.h>
-#include <stdint.h>
-
-/* Some BSDs require this for getrlimit */
-#include <sys/resource.h>
+/* asyncns.c removed this for #580301, but that appears to have been wrong */
+#if HAVE_ARPA_NAMESER_COMPAT_H
+#include <arpa/nameser_compat.h>
+#endif
 
 #include "asyncns.h"
 
