@@ -67,16 +67,16 @@ gboolean                        g_variant_is_of_type                    (GVarian
                                                                          const GVariantType   *type);
 gboolean                        g_variant_is_container                  (GVariant             *value);
 GVariantClass                   g_variant_classify                      (GVariant             *value);
-GVariant *                      g_variant_new_boolean                   (gboolean              boolean);
-GVariant *                      g_variant_new_byte                      (guchar                byte);
-GVariant *                      g_variant_new_int16                     (gint16                int16);
-GVariant *                      g_variant_new_uint16                    (guint16               uint16);
-GVariant *                      g_variant_new_int32                     (gint32                int32);
-GVariant *                      g_variant_new_uint32                    (guint32               uint32);
-GVariant *                      g_variant_new_int64                     (gint64                int64);
-GVariant *                      g_variant_new_uint64                    (guint64               uint64);
-GVariant *                      g_variant_new_handle                    (gint32                handle);
-GVariant *                      g_variant_new_double                    (gdouble               floating);
+GVariant *                      g_variant_new_boolean                   (gboolean              value);
+GVariant *                      g_variant_new_byte                      (guchar                value);
+GVariant *                      g_variant_new_int16                     (gint16                value);
+GVariant *                      g_variant_new_uint16                    (guint16               value);
+GVariant *                      g_variant_new_int32                     (gint32                value);
+GVariant *                      g_variant_new_uint32                    (guint32               value);
+GVariant *                      g_variant_new_int64                     (gint64                value);
+GVariant *                      g_variant_new_uint64                    (guint64               value);
+GVariant *                      g_variant_new_handle                    (gint32                value);
+GVariant *                      g_variant_new_double                    (gdouble               value);
 GVariant *                      g_variant_new_string                    (const gchar          *string);
 GVariant *                      g_variant_new_object_path               (const gchar          *object_path);
 gboolean                        g_variant_is_object_path                (const gchar          *string);
