@@ -169,56 +169,60 @@ static guint CrcTable[128];
  */
 static void crcinit(void)
 {
-        int i, j;
-        guint sum;
+  int i, j;
+  guint sum;
 
-        for (i = 0; i < 128; ++i) {
-                sum = 0L;
-                for (j = 7 - 1; j >= 0; --j)
-                        if (i & (1 << j))
-                                sum ^= POLY >> j;
-                CrcTable[i] = sum;
-        }
+  for (i = 0; i < 128; ++i)
+    {
+      sum = 0L;
+      for (j = 7 - 1; j >= 0; --j)
+        if (i & (1 << j))
+          sum ^= POLY >> j;
+      CrcTable[i] = sum;
+    }
 }
 
 /*
  - hash - Honeyman's nice hashing function
  */
-static guint honeyman_hash(gconstpointer key)
+static guint
+honeyman_hash (gconstpointer key)
 {
-        const gchar *name = (const gchar *) key;
-        gint size;
-        guint sum = 0;
+  const gchar *name = (const gchar *) key;
+  gint size;
+  guint sum = 0;
 
-        g_assert (name != NULL);
-        g_assert (*name != 0);
+  g_assert (name != NULL);
+  g_assert (*name != 0);
 
-        size = strlen(name);
+  size = strlen (name);
 
-        while (size--) {
-                sum = (sum >> 7) ^ CrcTable[(sum ^ (*name++)) & 0x7f];
-        }
+  while (size--)
+    sum = (sum >> 7) ^ CrcTable[(sum ^ (*name++)) & 0x7f];
 
-        return(sum);
+  return sum;
 }
 
 
-static gboolean second_hash_cmp (gconstpointer a, gconstpointer b)
+static gboolean
+second_hash_cmp (gconstpointer a, gconstpointer b)
 {
-  return (strcmp (a, b) == 0);
+  return strcmp (a, b) == 0;
 }
 
 
 
-static guint one_hash(gconstpointer key)
+static guint
+one_hash (gconstpointer key)
 {
   return 1;
 }
 
 
-static void not_even_foreach (gpointer       key,
-                                 gpointer       value,
-                                 gpointer       user_data)
+static void
+not_even_foreach (gpointer key,
+                  gpointer value,
+                  gpointer user_data)
 {
   const char *_key = (const char *) key;
   const char *_value = (const char *) value;
@@ -240,9 +244,10 @@ static void not_even_foreach (gpointer       key,
 }
 
 
-static gboolean remove_even_foreach (gpointer       key,
-                                 gpointer       value,
-                                 gpointer       user_data)
+static gboolean
+remove_even_foreach (gpointer key,
+                     gpointer value,
+                     gpointer user_data)
 {
   const char *_key = (const char *) key;
   const char *_value = (const char *) value;
@@ -265,174 +270,176 @@ static gboolean remove_even_foreach (gpointer       key,
 
 
 
-static void second_hash_test (gconstpointer d)
+static void
+second_hash_test (gconstpointer d)
 {
-     gboolean simple_hash = GPOINTER_TO_INT (d);
+  gboolean simple_hash = GPOINTER_TO_INT (d);
 
-     int       i;
-     char      key[20] = "", val[20]="", *v, *orig_key, *orig_val;
-     GHashTable     *h;
-     gboolean found;
+  int       i;
+  char      key[20] = "", val[20]="", *v, *orig_key, *orig_val;
+  GHashTable     *h;
+  gboolean found;
 
-     crcinit ();
+  crcinit ();
 
-     h = g_hash_table_new_full (simple_hash ? one_hash : honeyman_hash,
-                                second_hash_cmp,
-                                g_free, g_free);
-     g_assert (h != NULL);
-     for (i=0; i<20; i++)
-          {
-          sprintf (key, "%d", i);
-          g_assert (atoi (key) == i);
+  h = g_hash_table_new_full (simple_hash ? one_hash : honeyman_hash,
+                             second_hash_cmp,
+                             g_free, g_free);
+  g_assert (h != NULL);
+  for (i = 0; i < 20; i++)
+    {
+      sprintf (key, "%d", i);
+      g_assert (atoi (key) == i);
 
-          sprintf (val, "%d value", i);
-          g_assert (atoi (val) == i);
+      sprintf (val, "%d value", i);
+      g_assert (atoi (val) == i);
 
-          g_hash_table_insert (h, g_strdup (key), g_strdup (val));
-          }
+      g_hash_table_insert (h, g_strdup (key), g_strdup (val));
+    }
 
-     g_assert (g_hash_table_size (h) == 20);
+  g_assert (g_hash_table_size (h) == 20);
 
-     for (i=0; i<20; i++)
-          {
-          sprintf (key, "%d", i);
-          g_assert (atoi(key) == i);
+  for (i = 0; i < 20; i++)
+    {
+      sprintf (key, "%d", i);
+      g_assert (atoi(key) == i);
 
-          v = (char *) g_hash_table_lookup (h, key);
+      v = (char *) g_hash_table_lookup (h, key);
 
-          g_assert (v != NULL);
-          g_assert (*v != 0);
-          g_assert (atoi (v) == i);
-          }
+      g_assert (v != NULL);
+      g_assert (*v != 0);
+      g_assert (atoi (v) == i);
+    }
 
-     sprintf (key, "%d", 3);
-     g_hash_table_remove (h, key);
-     g_assert (g_hash_table_size (h) == 19);
-     g_hash_table_foreach_remove (h, remove_even_foreach, NULL);
-     g_assert (g_hash_table_size (h) == 9);
-     g_hash_table_foreach (h, not_even_foreach, NULL);
+  sprintf (key, "%d", 3);
+  g_hash_table_remove (h, key);
+  g_assert (g_hash_table_size (h) == 19);
+  g_hash_table_foreach_remove (h, remove_even_foreach, NULL);
+  g_assert (g_hash_table_size (h) == 9);
+  g_hash_table_foreach (h, not_even_foreach, NULL);
 
-     for (i=0; i<20; i++)
-          {
-          sprintf (key, "%d", i);
-          g_assert (atoi(key) == i);
+  for (i = 0; i < 20; i++)
+    {
+      sprintf (key, "%d", i);
+      g_assert (atoi(key) == i);
 
-          sprintf (val, "%d value", i);
-          g_assert (atoi (val) == i);
+      sprintf (val, "%d value", i);
+      g_assert (atoi (val) == i);
 
-          orig_key = orig_val = NULL;
-          found = g_hash_table_lookup_extended (h, key,
-                                                (gpointer)&orig_key,
-                                                (gpointer)&orig_val);
-          if ((i % 2) == 0 || i == 3)
-            {
-              g_assert (!found);
-              continue;
-            }
+      orig_key = orig_val = NULL;
+      found = g_hash_table_lookup_extended (h, key,
+                                            (gpointer)&orig_key,
+                                            (gpointer)&orig_val);
+      if ((i % 2) == 0 || i == 3)
+        {
+          g_assert (!found);
+          continue;
+        }
 
-          g_assert (found);
+      g_assert (found);
 
-          g_assert (orig_key != NULL);
-          g_assert (strcmp (key, orig_key) == 0);
+      g_assert (orig_key != NULL);
+      g_assert (strcmp (key, orig_key) == 0);
 
-          g_assert (orig_val != NULL);
-          g_assert (strcmp (val, orig_val) == 0);
-          }
+      g_assert (orig_val != NULL);
+      g_assert (strcmp (val, orig_val) == 0);
+    }
 
-    g_hash_table_destroy (h);
+  g_hash_table_destroy (h);
 }
 
-static gboolean find_first     (gpointer key, 
-                                gpointer value, 
-                                gpointer user_data)
+static gboolean
+find_first (gpointer key,
+            gpointer value,
+            gpointer user_data)
 {
-  gint *v = value; 
+  gint *v = value;
   gint *test = user_data;
   return (*v == *test);
 }
 
 
-static void direct_hash_test (void)
+static void
+direct_hash_test (void)
 {
-     gint       i, rc;
-     GHashTable     *h;
+  gint       i, rc;
+  GHashTable     *h;
 
-     h = g_hash_table_new (NULL, NULL);
-     g_assert (h != NULL);
-     for (i=1; i<=20; i++)
-          {
-          g_hash_table_insert (h, GINT_TO_POINTER (i),
-                               GINT_TO_POINTER (i + 42));
-          }
+  h = g_hash_table_new (NULL, NULL);
+  g_assert (h != NULL);
+  for (i = 1; i <= 20; i++)
+    g_hash_table_insert (h, GINT_TO_POINTER (i),
+                         GINT_TO_POINTER (i + 42));
 
-     g_assert (g_hash_table_size (h) == 20);
+  g_assert (g_hash_table_size (h) == 20);
 
-     for (i=1; i<=20; i++)
-          {
-          rc = GPOINTER_TO_INT (
-                g_hash_table_lookup (h, GINT_TO_POINTER (i)));
+  for (i = 1; i <= 20; i++)
+    {
+      rc = GPOINTER_TO_INT (g_hash_table_lookup (h, GINT_TO_POINTER (i)));
 
-          g_assert (rc != 0);
-          g_assert ((rc - 42) == i);
-          }
+      g_assert (rc != 0);
+      g_assert ((rc - 42) == i);
+    }
 
-    g_hash_table_destroy (h);
+  g_hash_table_destroy (h);
 }
 
-static void int64_hash_test (void)
+static void
+int64_hash_test (void)
 {
-     gint       i, rc;
-     GHashTable     *h;
-     gint64     values[20];
-     gint64 key;
+  gint       i, rc;
+  GHashTable     *h;
+  gint64     values[20];
+  gint64 key;
 
-     h = g_hash_table_new (g_int64_hash, g_int64_equal);
-     g_assert (h != NULL);
-     for (i=0; i<20; i++)
-          {
-          values[i] = i + 42;
-          g_hash_table_insert (h, &values[i], GINT_TO_POINTER (i + 42));
-          }
+  h = g_hash_table_new (g_int64_hash, g_int64_equal);
+  g_assert (h != NULL);
+  for (i = 0; i < 20; i++)
+    {
+      values[i] = i + 42;
+      g_hash_table_insert (h, &values[i], GINT_TO_POINTER (i + 42));
+    }
 
-     g_assert (g_hash_table_size (h) == 20);
+  g_assert (g_hash_table_size (h) == 20);
 
-     for (i=0; i<20; i++)
-          {
-          key = i + 42;
-          rc = GPOINTER_TO_INT (g_hash_table_lookup (h, &key));
+  for (i = 0; i < 20; i++)
+    {
+      key = i + 42;
+      rc = GPOINTER_TO_INT (g_hash_table_lookup (h, &key));
 
-          g_assert_cmpint (rc, ==, i + 42);
-          }
+      g_assert_cmpint (rc, ==, i + 42);
+    }
 
-    g_hash_table_destroy (h);
+  g_hash_table_destroy (h);
 }
 
-static void double_hash_test (void)
+static void
+double_hash_test (void)
 {
-     gint       i, rc;
-     GHashTable     *h;
-     gdouble values[20];
-     gdouble key;
+  gint       i, rc;
+  GHashTable     *h;
+  gdouble values[20];
+  gdouble key;
 
-     h = g_hash_table_new (g_double_hash, g_double_equal);
-     g_assert (h != NULL);
-     for (i=0; i<20; i++)
-          {
-          values[i] = i + 42.5;
-          g_hash_table_insert (h, &values[i], GINT_TO_POINTER (i + 42));
-          }
+  h = g_hash_table_new (g_double_hash, g_double_equal);
+  g_assert (h != NULL);
+  for (i = 0; i < 20; i++)
+    {
+      values[i] = i + 42.5;
+      g_hash_table_insert (h, &values[i], GINT_TO_POINTER (i + 42));
+    }
 
-     g_assert (g_hash_table_size (h) == 20);
+  g_assert (g_hash_table_size (h) == 20);
 
-     for (i=0; i<20; i++)
-          {
-          key = i + 42.5;
-          rc = GPOINTER_TO_INT (g_hash_table_lookup (h, &key));
+  for (i = 0; i < 20; i++)
+    {
+      key = i + 42.5;
+      rc = GPOINTER_TO_INT (g_hash_table_lookup (h, &key));
 
-          g_assert_cmpint (rc, ==, i + 42);
-          }
+      g_assert_cmpint (rc, ==, i + 42);
+    }
 
-    g_hash_table_destroy (h);
+  g_hash_table_destroy (h);
 }
 
 static void
@@ -443,41 +450,83 @@ string_free (gpointer data)
   g_string_free (s, TRUE);
 }
 
-static void string_hash_test (void)
+static void
+string_hash_test (void)
 {
-     gint       i, rc;
-     GHashTable     *h;
-     GString *s;
+  gint       i, rc;
+  GHashTable     *h;
+  GString *s;
 
-     h = g_hash_table_new_full ((GHashFunc)g_string_hash, (GEqualFunc)g_string_equal, string_free, NULL);
-     g_assert (h != NULL);
-     for (i=0; i<20; i++)
-          {
-          s = g_string_new ("");
-          g_string_append_printf (s, "%d", i + 42);
-          g_string_append_c (s, '.');
-          g_string_prepend_unichar (s, 0x2301);
-          g_hash_table_insert (h, s, GINT_TO_POINTER (i + 42));
-          }
+  h = g_hash_table_new_full ((GHashFunc)g_string_hash, (GEqualFunc)g_string_equal, string_free, NULL);
+  g_assert (h != NULL);
+  for (i = 0; i < 20; i++)
+    {
+      s = g_string_new ("");
+      g_string_append_printf (s, "%d", i + 42);
+      g_string_append_c (s, '.');
+      g_string_prepend_unichar (s, 0x2301);
+      g_hash_table_insert (h, s, GINT_TO_POINTER (i + 42));
+    }
 
-     g_assert (g_hash_table_size (h) == 20);
+  g_assert (g_hash_table_size (h) == 20);
 
-     s = g_string_new ("");
-     for (i=0; i<20; i++)
-          {
-          g_string_assign (s, "");
-          g_string_append_printf (s, "%d", i + 42);
-          g_string_append_c (s, '.');
-          g_string_prepend_unichar (s, 0x2301);
-          rc = GPOINTER_TO_INT (g_hash_table_lookup (h, s));
+  s = g_string_new ("");
+  for (i = 0; i < 20; i++)
+    {
+      g_string_assign (s, "");
+      g_string_append_printf (s, "%d", i + 42);
+      g_string_append_c (s, '.');
+      g_string_prepend_unichar (s, 0x2301);
+      rc = GPOINTER_TO_INT (g_hash_table_lookup (h, s));
 
-          g_assert_cmpint (rc, ==, i + 42);
-          }
+      g_assert_cmpint (rc, ==, i + 42);
+    }
 
-    g_string_free (s, TRUE);
-    g_hash_table_destroy (h);
+  g_string_free (s, TRUE);
+  g_hash_table_destroy (h);
 }
 
+static void
+set_check (gpointer key,
+           gpointer value,
+           gpointer user_data)
+{
+  int *pi = user_data;
+  if (key != value)
+    g_assert_not_reached ();
+
+  g_assert_cmpint (atoi (key) % 7, ==, 2);
+
+  (*pi)++;
+}
+
+static void
+set_hash_test (void)
+{
+  GHashTable *hash_table =
+    g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+  int i;
+
+  for (i = 2; i < 5000; i += 7)
+    {
+      char *s = g_strdup_printf ("%d", i);
+      g_hash_table_insert (hash_table, s, s);
+    }
+
+  i = 0;
+  g_hash_table_foreach (hash_table, set_check, &i);
+  g_assert_cmpint (i, ==, g_hash_table_size (hash_table));
+
+  /* this will cause the hash table to loose set nature */
+  g_hash_table_insert (hash_table, g_strdup ("a"), "b");
+
+  g_assert_cmpstr (g_hash_table_lookup (hash_table, "2"), ==, "2");
+  g_assert_cmpstr (g_hash_table_lookup (hash_table, "a"), ==, "b");
+
+  g_hash_table_destroy (hash_table);
+}
+
+
 static void
 test_hash_misc (void)
 {
@@ -648,6 +697,134 @@ test_lookup_null_key (void)
   g_hash_table_unref (h);
 }
 
+static gint destroy_key_counter;
+
+static void
+key_destroy (gpointer key)
+{
+  destroy_key_counter++;
+}
+
+static void
+test_remove_all (void)
+{
+  GHashTable *h;
+
+  h = g_hash_table_new_full (g_str_hash, g_str_equal, key_destroy, value_destroy);
+  g_hash_table_insert (h, "abc", "ABC");
+  g_hash_table_insert (h, "cde", "CDE");
+  g_hash_table_insert (h, "xyz", "XYZ");
+
+  destroy_counter = 0;
+  destroy_key_counter = 0;
+
+  g_hash_table_steal_all (h);
+  g_assert_cmpint (destroy_counter, ==, 0);
+  g_assert_cmpint (destroy_key_counter, ==, 0);
+
+  g_hash_table_insert (h, "abc", "ABC");
+  g_hash_table_insert (h, "cde", "CDE");
+  g_hash_table_insert (h, "xyz", "XYZ");
+
+  g_hash_table_remove_all (h);
+  g_assert_cmpint (destroy_counter, ==, 3);
+  g_assert_cmpint (destroy_key_counter, ==, 3);
+
+  g_hash_table_unref (h);
+}
+
+typedef struct {
+  gint ref_count;
+  const gchar *key;
+} RefCountedKey;
+
+static guint
+hash_func (gconstpointer key)
+{
+  const RefCountedKey *rkey = key;
+
+  return g_str_hash (rkey->key);
+}
+
+static gboolean
+eq_func (gconstpointer a, gconstpointer b)
+{
+  const RefCountedKey *aa = a;
+  const RefCountedKey *bb = b;
+
+  return g_strcmp0 (aa->key, bb->key) == 0;
+}
+
+static void
+key_unref (gpointer data)
+{
+  RefCountedKey *key = data;
+
+  g_assert (key->ref_count > 0);
+
+  key->ref_count -= 1;
+
+  if (key->ref_count == 0)
+    g_free (key);
+}
+
+static RefCountedKey *
+key_ref (RefCountedKey *key)
+{
+  key->ref_count += 1;
+
+  return key;
+}
+
+static RefCountedKey *
+key_new (const gchar *key)
+{
+  RefCountedKey *rkey;
+
+  rkey = g_new (RefCountedKey, 1);
+
+  rkey->ref_count = 1;
+  rkey->key = key;
+
+  return rkey;
+}
+
+static void
+set_ref_hash_test (void)
+{
+  GHashTable *h;
+  RefCountedKey *key1;
+  RefCountedKey *key2;
+
+  h = g_hash_table_new_full (hash_func, eq_func, key_unref, key_unref);
+
+  key1 = key_new ("a");
+  key2 = key_new ("a");
+
+  g_assert_cmpint (key1->ref_count, ==, 1);
+  g_assert_cmpint (key2->ref_count, ==, 1);
+
+  g_hash_table_insert (h, key_ref (key1), key_ref (key1));
+
+  g_assert_cmpint (key1->ref_count, ==, 3);
+  g_assert_cmpint (key2->ref_count, ==, 1);
+
+  g_hash_table_replace (h, key_ref (key2), key_ref (key2));
+
+  g_assert_cmpint (key1->ref_count, ==, 1);
+  g_assert_cmpint (key2->ref_count, ==, 3);
+
+  g_hash_table_remove (h, key1);
+
+  g_assert_cmpint (key1->ref_count, ==, 1);
+  g_assert_cmpint (key2->ref_count, ==, 1);
+
+  g_hash_table_unref (h);
+
+  key_unref (key1);
+  key_unref (key2);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -662,7 +839,10 @@ main (int argc, char *argv[])
   g_test_add_func ("/hash/int64", int64_hash_test);
   g_test_add_func ("/hash/double", double_hash_test);
   g_test_add_func ("/hash/string", string_hash_test);
+  g_test_add_func ("/hash/set", set_hash_test);
+  g_test_add_func ("/hash/set-ref", set_ref_hash_test);
   g_test_add_func ("/hash/ref", test_hash_ref);
+  g_test_add_func ("/hash/remove-all", test_remove_all);
 
   /* tests for individual bugs */
   g_test_add_func ("/hash/lookup-null-key", test_lookup_null_key);
